@@ -2,78 +2,139 @@
  * Объектное дерево, включает в себя тулбар с кнопками добавить (в корень и дочерний элемент), редактировать и удалить
  * @param {Object} config
  */
-Ext.m3.ObjectTree = Ext.extend(Ext.ux.tree.TreeGrid, {
-    constructor: function (baseConfig, params) {
-        assert(params.rowIdName !== undefined, 'rowIdName is undefined');
-        assert(params.actions !== undefined, 'actions is undefined');
 
-        this.rootVisible = params.rootVisible;
-        this.allowPaging = params.allowPaging;
-        this.rowIdName = params.rowIdName;
-        this.actionNewUrl = params.actions.newUrl;
-        this.actionEditUrl = params.actions.editUrl;
-        this.actionDeleteUrl = params.actions.deleteUrl;
-        this.actionDataUrl = params.actions.dataUrl;
-        this.actionContextJson = params.actions.contextJson;
-        this.parentIdName = params.parentIdName;
-        this.incrementalUpdate = params.incrementalUpdate;
-        if (params.customLoad) {
-            var ajax = Ext.Ajax;
-            this.on('expandnode', function (node) {
-                var nodeList = [];
-                if (node.hasChildNodes()) {
-                    for (var i = 0; i < node.childNodes.length; i++) {
-                        if (!node.childNodes[i].isLoaded()) {
-                            nodeList.push(node.childNodes[i].id);
-                        }
-                    }
-                }
-                if (nodeList.length > 0) {
-                    ajax.request({
-                        url: params.actions.dataUrl,
-                        params: {'list_nodes': nodeList.join(',')},
-                        success: function (response, opts) {
-                            var res = Ext.util.JSON.decode(response.responseText);
-                            if (res) {
-                                for (var i = 0; i < res.length; i++) {
-                                    var curr_node = node.childNodes[i];
-                                    for (var j = 0; j < res[i].children.length; j++) {
-                                        var newNode = new Ext.tree.AsyncTreeNode(res[i].children[j]);
-                                        curr_node.appendChild(newNode);
-                                        curr_node.loaded = true;
-                                    }
-                                }
-                            }
-                        }, failure: function (response, opts) {
-                            Ext.Msg.alert('', 'failed');
-                        }
-                    });
+Ext.define('Ext.m3.ObjectTree', {
+    extend: 'Ext.m3.Tree',
+    xtype: 'm3-object-tree',
+
+    bubbleEvents: [
+        'mask',
+        'unmask',
+        'getcontext'
+    ],
+
+    allowPaging: false,
+    rowIdName: 'id',
+    parentIdName: 'parent_id',
+
+    actionNewUrl: null,
+    actionEditUrl: null,
+    actionDeleteUrl: null,
+
+    incrementalUpdate: false,
+
+    folderSort: true,
+    enableSort: false,
+
+    configure: function () {
+
+        var contextMenu = Ext.create({}, 'menu'),
+            containerContextMenu = Ext.create({}, 'menu'),
+            tbar = Ext.create(this.tbar),
+            buff,
+
+            buttonNewInRoot = {
+                text: 'Новый в корне',
+                iconCls: 'add_item',
+                handler: this.onNewRecord,
+                scope: this
+            },
+            buttonNewInChild = {
+                text: 'Новый дочерний',
+                iconCls: 'add_item',
+                handler: this.onNewRecordChild,
+                scope: this
+            },
+            buttonEdit = {
+                text: 'Изменить',
+                iconCls: 'edit_item',
+                handler: this.onEditRecord,
+                scope: this
+            },
+            buttonRemove = {
+                text: 'Удалить',
+                iconCls: 'delete_item',
+                handler: this.onDeleteRecord,
+                scope: this
+            },
+            buttonRefresh = {
+                text: 'Обновить',
+                iconCls: 'x-tbar-loading',
+                handler: this.refreshStore,
+                scope: this
+            };
+
+        // сохраняем пользовательские кнопки
+        buff = tbar.removeAll(false);
+
+        if (this.actionNewUrl) {
+            contextMenu.add(buttonNewInRoot);
+            contextMenu.add(buttonNewInChild);
+            containerContextMenu.add(buttonNewInRoot);
+
+            tbar.add({
+                text: 'Добавить',
+                iconCls: 'add_item',
+                menu: {
+                    items: [buttonNewInRoot, buttonNewInChild]
                 }
+
             });
+
         }
-        // Параметр "Сортировать папки"
-        // если true, то папки всегда будут выше простых элементов
-        // если false, то папки ведут себя также как элементы
-        baseConfig.folderSort = true;
-        baseConfig.enableSort = false;
-        if (params.folderSort != undefined) {
-            baseConfig.folderSort = params.folderSort;
+        if (this.actionEditUrl) {
+            contextMenu.add(buttonEdit);
+            tbar.add(buttonEdit);
+
+            this.on('dblclick', this.onEditRecord, this);
         }
-        Ext.m3.ObjectTree.superclass.constructor.call(this, baseConfig, params);
+        if (this.actionDeleteUrl) {
+            contextMenu.add(buttonRemove);
+            tbar.add(buttonRemove);
+        }
+
+        // add separator
+        if (this.actionNewUrl || this.actionEditUrl || this.actionDeleteUrl) {
+            tbar.add('-');
+            contextMenu.add('-');
+            containerContextMenu.add('-');
+        }
+
+        if (this.dataUrl) {
+            tbar.add(buttonRefresh);
+            contextMenu.add(buttonRefresh);
+            containerContextMenu.add(buttonRefresh);
+        }
+
+        if (contextMenu.items && contextMenu.items.length > 0) {
+            this.contextMenu = contextMenu;
+        }
+        if (containerContextMenu.items && containerContextMenu.items.length > 0) {
+            this.containerContextMenu = containerContextMenu;
+        }
+
+        // восстанавливаем пользовательские кнопки
+        tbar.add(buff);
+
+        this.tbar = tbar;
     },
 
     initComponent: function () {
-        var loader = this.getLoader();
-        loader.baseParams = this.getMainContext();
 
-        Ext.m3.ObjectTree.superclass.initComponent.call(this);
-        // Созадем свой сортировщик с переданными параметрами
-        var sorter = new Ext.ux.tree.TreeGridSorter(this, {folderSort: this.folderSort,
-            property: this.columns[0].dataIndex || 'text'});
+        this.configure();
+
+        this.callParent();
+
+        var loader = this.getLoader();
+            _self = this;
+        loader.on('beforeload', function(ldr) {
+            ldr.baseParams = Ext.apply(ldr.baseParams || {}, _self.getContext());
+        });
         // Повесим отображение маски при загрузке дерева
-        loader.on('beforeload', this.onBeforeLoad, this);
-        loader.on('load', this.onLoad, this);
-        loader.on('loadexception', this.onLoadException, this);
+        loader.on('beforeload', this.fireEvent.createDelegate(this, ['mask', this, 'Загрузка...']), this);
+        loader.on('load', this.fireEvent.createDelegate(this, ['unmask', this]), this);
+        loader.on('loadexception', this.fireEvent.createDelegate(this, ['unmask', this]), this);
+
         // еще настроим loader, чтобы правильно передавал узел через параметр
         loader.nodeParameter = this.rowIdName;
 
@@ -102,48 +163,19 @@ Ext.m3.ObjectTree = Ext.extend(Ext.ux.tree.TreeGrid, {
         );
     },
 
-    showMask: function (visible) {
-        var loader = this.getLoader();
-        if (this.treeLoadingMask == undefined) {
-            this.treeLoadingMask = new Ext.LoadMask(this.el, {msg: "Загрузка..."});
-        }
-        if (visible) {
-            this.treeLoadingMask.show();
-        } else {
-            this.treeLoadingMask.hide();
-        }
-    },
-
-    onBeforeLoad: function (treeloader, node, callback) {
-        this.showMask(true);
-    },
-
-    onLoad: function (treeloader, node, response) {
-        this.showMask(false);
-    },
-
-    onLoadException: function (treeloader, node, response) {
-        this.showMask(false);
-    },
 
     onNewRecord: function () {
         assert(this.actionNewUrl, 'actionNewUrl is not define');
 
-        var req = {
+        var request = {
             url: this.actionNewUrl,
-            method: 'POST',
-            params: this.getMainContext(),
-            scope: this,
-            success: function (res, opt) {
-                return this.childWindowOpenHandler(res, opt, 'new');
-            },
-            failure: function () {
-                uiAjaxFailMessage.apply(this, arguments);
-            }
+            params: this.getContext(),
+            success: this.childWindowOpenHandler.createDelegate(this, ['new'], true),
+            failure: uiAjaxFailMessage,
+            mode: "Режим создания..."
         };
-
-        if (this.fireEvent('beforenewrequest', this, req, false)) {
-            Ext.Ajax.request(req);
+        if (this.fireEvent('beforenewrequest', this, request)) {
+            UI.callAction.call(this, request);
         }
     },
 
@@ -162,24 +194,19 @@ Ext.m3.ObjectTree = Ext.extend(Ext.ux.tree.TreeGrid, {
         var baseConf = this.getSelectionContext();
         baseConf[this.parentIdName] = baseConf[this.rowIdName];
         delete baseConf[this.rowIdName];
-        var scope = this;
 
-        var req = {
+
+        var request = {
             url: this.actionNewUrl,
-            scope: this,
-            method: "POST",
             params: baseConf,
-            success: function (res, opt) {
-                return this.childWindowOpenHandler(res, opt, 'newChild');
-            },
-            failure: function () {
-                uiAjaxFailMessage.apply(this, arguments);
-            }
+            success: this.childWindowOpenHandler.createDelegate(this, ['newChild'], true),
+            failure: uiAjaxFailMessage,
+            mode: "Режим создания..."
         };
-
-        if (this.fireEvent('beforenewrequest', this, req, true)) {
-            Ext.Ajax.request(req);
+        if (this.fireEvent('beforenewrequest', this, request)) {
+            UI.callAction.call(this, request);
         }
+
     },
 
     onEditRecord: function () {
@@ -187,63 +214,50 @@ Ext.m3.ObjectTree = Ext.extend(Ext.ux.tree.TreeGrid, {
         assert(this.rowIdName, 'rowIdName is not define');
 
         if (this.getSelectionModel().getSelectedNode()) {
-            var baseConf = this.getSelectionContext();
 
-            var req = {
+            var request = {
                 url: this.actionEditUrl,
-                scope: this,
-                method: 'POST',
-                params: baseConf,
-                success: function (res, opt) {
-                    return this.childWindowOpenHandler(res, opt, 'edit');
-                },
-                failure: function () {
-                    uiAjaxFailMessage.apply(this, arguments);
-                }
+                params: this.getSelectionContext(),
+                success: this.childWindowOpenHandler.createDelegate(this, ['edit'], true),
+                failure: uiAjaxFailMessage,
+                mode: "Режим редактирования..."
             };
-
-            if (this.fireEvent('beforeeditrequest', this, req)) {
-                Ext.Ajax.request(req);
+            if (this.fireEvent('beforeeditrequest', this, request)) {
+                UI.callAction.call(this, request);
             }
+
         }
     },
 
     onDeleteRecord: function () {
         assert(this.actionDeleteUrl, 'actionDeleteUrl is not define');
         assert(this.rowIdName, 'rowIdName is not define');
-        var node = this.getSelectionModel().getSelectedNode()
+        var node = this.getSelectionModel().getSelectedNode();
         if (node) {
 
             Ext.Msg.show({
                 title: 'Удаление записи',
-                scope: this,
                 msg: 'Вы действительно хотите удалить выбранную запись?',
                 icon: Ext.Msg.QUESTION,
                 buttons: Ext.Msg.YESNO,
-                fn: function (btn, text, opt) {
+                fn: function (btn) {
                     if (btn != 'yes')
                         return;
 
                     if (this.getSelectionModel().getSelectedNode()) {
-                        var baseConf = this.getSelectionContext();
 
-                        var req = {
+                        var request = {
                             url: this.actionDeleteUrl,
-                            scope: this,
-                            params: baseConf,
-                            success: function (res, opt) {
-                                return this.deleteOkHandler(res, opt);
-                            },
-                            failure: function () {
-                                uiAjaxFailMessage.apply(this, arguments);
-                            }
+                            method: 'POST',
+                            params: this.getSelectionContext(),
+                            success: this.deleteOkHandler.createDelegate(this),
+                            failure: uiAjaxFailMessage
                         };
-
-                        if (this.fireEvent('beforedeleterequest', this, req)) {
-                            Ext.Ajax.request(req);
+                        if (this.fireEvent('beforedeleterequest', this, request)) {
+                            UI.callAction.call(this, request);
                         }
                     }
-                }
+                }.bind(this)
             });
         } else {
             Ext.Msg.show({
@@ -255,15 +269,13 @@ Ext.m3.ObjectTree = Ext.extend(Ext.ux.tree.TreeGrid, {
         }
     },
 
-    childWindowOpenHandler: function (response, opts, operation) {
-
-        var window = smart_eval(response.responseText);
-        if (window) {
-            window.on('closed_ok', function (data) {
+    childWindowOpenHandler: function (win, operation) {
+        if (win) {
+            win.on('closed_ok', function (data) {
                 if (this.incrementalUpdate) {
                     // нам пришел узел дерева
-                    var obj = Ext.util.JSON.decode(data);
-                    var selectedNode = this.getSelectionModel().getSelectedNode();
+                    var obj = Ext.decode(data),
+                        selectedNode = this.getSelectionModel().getSelectedNode();
                     var newSelectNode = this.getLoader().createNode(obj.data);
                     switch (operation) {
                         case 'edit':
@@ -320,17 +332,15 @@ Ext.m3.ObjectTree = Ext.extend(Ext.ux.tree.TreeGrid, {
                     }
                 }
                 else {
-                    return this.refreshStore()
+                    this.refreshStore();
                 }
             }, this);
         }
+        return win;
     },
-    deleteOkHandler: function (response, opts) {
+    deleteOkHandler: function (res) {
         if (this.incrementalUpdate) {
-            // проверка на ошибки уровня приложения
-            var res = Ext.util.JSON.decode(response.responseText);
             if (!res.success) {
-                smart_eval(response.responseText);
                 return;
             }
             // нам просто надо удалить выделенный элемент
@@ -339,33 +349,22 @@ Ext.m3.ObjectTree = Ext.extend(Ext.ux.tree.TreeGrid, {
             parentNode.removeChild(selectedNode);
             parentNode.select();
         } else {
-            smart_eval(response.responseText);
             this.refreshStore();
         }
     },
     refreshStore: function () {
-        this.getLoader().baseParams = this.getMainContext();
+        this.getLoader().baseParams = Ext.apply({}, this.getContext());
         this.getLoader().load(this.getRootNode());
     },
-    /**
-     * Получение основного контекста дерева
-     * Используется при ajax запросах
-     */
-    getMainContext: function () {
-        return Ext.applyIf({}, this.actionContextJson);
-    },
     /**
      * Получение контекста выделения строк/ячеек
      * Используется при ajax запросах
-     * @param {bool} withRow Признак добавление в контекст текущей выбранной записи
      */
-    getSelectionContext: function (withRow) {
-        var baseConf = this.getMainContext();
+    getSelectionContext: function () {
+        var baseConf = Ext.apply({}, this.getContext());
         if (this.getSelectionModel().getSelectedNode()) {
             baseConf[this.rowIdName] = this.getSelectionModel().getSelectedNode().id;
         }
         return baseConf;
     }
 });
-
-Ext.reg('m3-object-tree', Ext.m3.ObjectTree);
